#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <pwd.h>
#include <unistd.h>
#include <selinux/selinux.h>

#include <QApplication>
#include <QDebug>
#include <QTranslator>
#include <QLibraryInfo>

#include "mainwindow.h"

//define DEBUG_CHECK 1

extern const char *__progname;

int main( int argc, char **argv) {

    int ret = 0;
    QApplication app(argc, argv);
    app.setWindowIcon(QIcon::fromTheme(QStringLiteral("basealt"),QIcon::fromTheme(QStringLiteral("altlinux"))));

    QTranslator *qt_translator = new QTranslator(&app);
    QTranslator *app_translator = new QTranslator(&app);
    qt_translator->load(QLibraryInfo::location(QLibraryInfo::TranslationsPath) + QStringLiteral("/qtbase_") + QLocale::system().name());
    app_translator->load(QLibraryInfo::location(QLibraryInfo::TranslationsPath) + QStringLiteral("/sechooser_") + QLocale::system().name());
    QCoreApplication::installTranslator(qt_translator);
    QCoreApplication::installTranslator(app_translator);

#ifdef DEBUG_CHECK
    if( true ) {
#else
    if( is_selinux_mls_enabled() > 0 ) {
#endif
	char *seuser = NULL;
	char *urange = NULL;
#define CHARS_IN_UID_MAX 255
	char uidbuf[CHARS_IN_UID_MAX+1];
	int  uidfile = 0;
	uid_t myuid = 0;
	struct passwd *login = NULL;
	int bytes_read_uid=0;
	uidfile=open("/proc/self/loginuid",O_RDONLY);
	if (uidfile == -1) {
		perror("cannot open uid in proc ");
		exit(1);
	}
        bytes_read_uid=read(uidfile,uidbuf,CHARS_IN_UID_MAX);
	if (bytes_read_uid<=0) {
		perror("cannot read loginuid ");
		close(uidfile);
		exit(1);
	}
        close(uidfile);
	myuid=(uid_t)atoll(uidbuf);
	errno=0;
        login=getpwuid(myuid);
	if(login == NULL) {
		perror(" no login name for current uid" );
		exit(1);
	}

        if( getseuserbyname(login->pw_name, &seuser, &urange) == 0 ) {

#ifdef DEBUG_CHECK
	    urange = strdup("s0:c1,c3,c6.c8-s5:c1.c8,c10,c12");
#endif
	    qDebug("%s: selinux level<%s>", __progname, urange);
	    QLatin1String init_urange(urange);
#ifdef DONT_IGNORE_LOWEST_LEVEL
	    if( true ) {
#else
	    if( init_urange != QStringLiteral("s0-s0") && init_urange != QStringLiteral("s0") ) {
#endif
		MainWindow main_window;
		main_window.setRange(init_urange);
		main_window.show();
		main_window.activateWindow();

		ret = app.exec();
	    } else {
		ret = 0;
	    }
	    free(seuser);
	    free(urange);
	} else {
	    qWarning("%s: unable to get user level", __progname);
	    ret = 1;
	}
    } else {
	qWarning("%s: selinux mls disabled", __progname);
    }
    return ret;
}
